@@ -8,13 +8,13 @@
 
 namespace at::native {
 
-bool is_pinned_cuda(const Tensor& self, c10::optional<Device> device) {
+bool is_pinned_cuda(const Tensor& self, std::optional<Device> device) {
   TORCH_INTERNAL_ASSERT_DEBUG_ONLY(!device.has_value() || device->is_cuda());
   // TODO: unhook this
   return detail::getCUDAHooks().isPinnedPtr(self.storage().data());
 }
 
-Tensor _pin_memory_cuda(const Tensor& self, c10::optional<Device> device) {
+Tensor _pin_memory_cuda(const Tensor& self, std::optional<Device> device) {
   TORCH_INTERNAL_ASSERT_DEBUG_ONLY(!device.has_value() || device->is_cuda());
   auto* allocator = at::cuda::getPinnedMemoryAllocator();
   auto storage = Storage(
