@@ -147,21 +147,18 @@ TORCH_META_FUNC(_convert_indices_from_csr_to_coo)
  const bool out_int32,
  const bool transpose) {
   TORCH_CHECK(
-    crow_indices.dim() == 1, "crow_indices is supposed to be a vector, but got ",
-    crow_indices.dim(), " dimensional tensor.");
-  TORCH_CHECK(col_indices.dim() == 1, "col_indices is supposed to be a vector, but got ",
-              col_indices.dim(), " dimensional tensor.");
+    crow_indices.dim() == col_indices.dim(), "crow_indices and col_indices are supposed to have"
+    " the same dimensionality, but got ", crow_indices.dim(), " and ",
+    crow_indices.dim(), " dimensional tensors, respectively.");
   ScalarType scalar_type = out_int32 ? ScalarType::Int : ScalarType::Long;
   c10::TensorOptions options = crow_indices.options().dtype(scalar_type);
-  set_output_raw_strided(0, {2, col_indices.numel()}, {}, options, {});
+  set_output_raw_strided(0, {col_indices.dim() + 1, col_indices.numel()}, {}, options, {});
 }
 
 } // namespace meta
 
 namespace {
 
-constexpr int64_t GRAIN_SIZE = at::internal::GRAIN_SIZE;
-
 template <typename F>
 Tensor& unary_op_out(F op_out, const Tensor& self, Tensor& result) {
   TORCH_INTERNAL_ASSERT(self.is_sparse_csr());
@@ -194,40 +191,12 @@ Tensor& unary_op_inplace(Tensor& self, const F& op_inplace, Args&&... args) {
   return self;
 }
 
-template <typename input_t, typename output_t>
-void convert_indices_from_csr_to_coo_cpu(
-    const Tensor& indices,
-    const Tensor& crow_indices,
-    const Tensor& col_indices,
-    const bool transpose = false) {
-  int64_t nrows = crow_indices.numel() - 1;
-  if (nrows == 0) {
-    indices.zero_();
-    return;
-  }
-  auto crow_indices_ = crow_indices.expect_contiguous();
-  const input_t* crow_indices_data_in = crow_indices_->data_ptr<input_t>();
-  TORCH_INTERNAL_ASSERT(indices.is_contiguous());
-  auto row0 = indices.select(0, transpose ? 1 : 0);
-  auto row1 = indices.select(0, transpose ? 0 : 1);
-  output_t* data_out = row0.data_ptr<output_t>();
-  row1.copy_(*col_indices.expect_contiguous());
-  at::parallel_for(0, nrows, GRAIN_SIZE, [&](int64_t start, int64_t end) {
-    for (const auto i : c10::irange(start, end)) {
-      std::fill(
-          &data_out[crow_indices_data_in[i]],
-          &data_out[crow_indices_data_in[i + 1]],
-          static_cast<output_t>(i));
-    }
-  });
-}
-
 } // end anonymous namespace
 
 namespace native {
 
 using namespace at::sparse_csr;
-// certain utiliy functions are usable from sparse COO.
+// certain utility functions are usable from sparse COO.
 using namespace at::sparse;
 
 Tensor& mul_out_sparse_csr(const Tensor& t_, const Tensor& src_, Tensor& r) {
@@ -253,16 +222,13 @@ Tensor intersection_binary_op_with_wrapped_scalar(const Tensor& sparse, const Te
   // NOTE: intersection_binary_op_with_wrapped_scalar assumes scalar.numel() == 1.
   const auto result_values = op(sparse.values(), scalar.squeeze()).to(at::result_type(sparse, scalar));
   const auto result_sizes = infer_size(sparse.sizes(), scalar.sizes());
-  Tensor compressed_indices, plain_indices;
-  std::tie(compressed_indices, plain_indices) = getCompressedPlainIndices(sparse);
-  return at::native::_sparse_compressed_tensor_unsafe(
+  auto [compressed_indices, plain_indices] = getCompressedPlainIndices(sparse);
+  return at::_sparse_compressed_tensor_unsafe(
       compressed_indices.clone(),
       plain_indices.clone(),
       result_values,
       result_sizes,
-      result_values.scalar_type(),
-      sparse.layout(),
-      result_values.device());
+      sparse.options().dtype(result_values.scalar_type()));
 }
 
 template <typename op_t>
@@ -337,14 +303,12 @@ inline Tensor get_result_tensor_for_unary_op(F op, const Tensor& input) {
                                                                  [&]{ return input.col_indices(); },
                                                                  [&]{ return input.row_indices(); });
 
-  auto result = at::native::_sparse_compressed_tensor_unsafe(
+  auto result = at::_sparse_compressed_tensor_unsafe(
       compressed_indices.clone(),
       plain_indices.clone(),
       result_values,
       input.sizes(),
-      result_values.scalar_type(),
-      input.layout(),
-      result_values.device());
+      input.options().dtype(result_values.scalar_type()));
 
   return result;
 }
@@ -362,7 +326,7 @@ Tensor& normal_sparse_csr_(
     Tensor& self,
     double mean,
     double std,
-    c10::optional<Generator> gen) {
+    std::optional<Generator> gen) {
   return unary_op_inplace(self, &Tensor::normal_, mean, std, gen);
 }
 
@@ -391,17 +355,14 @@ Tensor sparse_mask_sparse_compressed(
   }
 
   if (self.layout() == kStrided) {
-    Tensor compressed_indices, plain_indices;
-    std::tie(compressed_indices, plain_indices) = at::sparse_csr::getCompressedPlainIndices(mask);
+    auto [compressed_indices, plain_indices] = at::sparse_csr::getCompressedPlainIndices(mask);
     auto mask_values = mask.values();
-    auto dense_mask = at::native::_sparse_compressed_tensor_unsafe(
+    auto dense_mask = at::_sparse_compressed_tensor_unsafe(
         compressed_indices,
         plain_indices,
         at::ones({1}, self.options().dtype(kBool)).expand_as(mask_values),
         self.sizes(),
-        kBool,
-        mask.layout(),
-        self.device()).to_dense();
+        self.options().dtype(kBool).layout(mask.layout())).to_dense();
     return AT_DISPATCH_PLAIN_SPARSE_COMPRESSED_LAYOUTS(
         mask.layout(), "sparse_mask_sparse_compressed",
         [&] {
@@ -831,7 +792,22 @@ Tensor add_sparse_csr(
     const Scalar& alpha) {
   auto commonDtype = at::result_type(self, other);
   alpha_check(commonDtype, alpha);
-  Tensor result = at::empty_like(self, self.options().dtype(commonDtype).memory_format(at::MemoryFormat::Contiguous));
+  Tensor result;
+  if (self.layout() != kStrided && other.layout() == kStrided) {
+    // add(sparse, dense) -> dense
+    result = at::empty_like(
+        other,
+        other.options()
+            .dtype(commonDtype)
+            .memory_format(at::MemoryFormat::Contiguous));
+  } else {
+    // add(dense, sparse) -> dense AND add(sparse, sparse) -> sparse
+    result = at::empty_like(
+        self,
+        self.options()
+            .dtype(commonDtype)
+            .memory_format(at::MemoryFormat::Contiguous));
+  }
   return at::add_out(result, self, other, alpha); // redispatch!
 }
 
@@ -842,13 +818,14 @@ Tensor& add_sparse_csr_(
   return at::add_out(self, self, other, alpha); // redispatch!
 }
 
-static void add_out_dense_sparse_csr_cpu(
+static void add_out_dense_sparse_compressed_cpu(
     const Tensor& out,
     const Tensor& dense,
     const SparseCsrTensor& src,
     const Scalar& alpha) {
   TORCH_INTERNAL_ASSERT(dense.layout() == kStrided);
-  TORCH_INTERNAL_ASSERT(src.is_sparse_csr());
+  TORCH_INTERNAL_ASSERT(
+      src.layout() == kSparseCsr || src.layout() == kSparseCsc);
   TORCH_INTERNAL_ASSERT(dense.device() == kCPU);
 
   TORCH_CHECK(
@@ -899,8 +876,15 @@ static void add_out_dense_sparse_csr_cpu(
 
   auto valuesBuffer = src_values.to(commonDtype).reshape({-1, src_values.size(-1)});
   resultBuffer = resultBuffer.view({-1, out.size(-2), out.size(-1)});
-  auto src_crow_indices = src.crow_indices().reshape({-1, src.crow_indices().size(-1)});
-  auto src_col_indices = src.col_indices().reshape({-1, src.col_indices().size(-1)});
+  Tensor src_compressed_indices;
+  Tensor src_plain_indices;
+  std::tie(src_compressed_indices, src_plain_indices) =
+      at::sparse_csr::getCompressedPlainIndices(src);
+  src_compressed_indices =
+      src_compressed_indices.reshape({-1, src_compressed_indices.size(-1)});
+  src_plain_indices =
+      src_plain_indices.reshape({-1, src_plain_indices.size(-1)});
+  auto src_layout = src.layout();
 
   AT_DISPATCH_ALL_TYPES_AND_COMPLEX_AND4(
       kComplexHalf,
@@ -912,35 +896,57 @@ static void add_out_dense_sparse_csr_cpu(
       [&valuesBuffer,
        &resultBuffer,
        &alpha,
-       &src_crow_indices,
-       &src_col_indices]() {
+       &src_compressed_indices,
+       &src_plain_indices,
+       &src_layout]() {
         AT_DISPATCH_INDEX_TYPES(
-            src_crow_indices.scalar_type(),
+            src_compressed_indices.scalar_type(),
             "csr_add_out_crow_indices",
             [&valuesBuffer,
              &resultBuffer,
              &alpha,
-             &src_crow_indices,
-             &src_col_indices]() {
-              auto batch_count = resultBuffer.dim() > 2 ? resultBuffer.size(-3) : 1;
+             &src_compressed_indices,
+             &src_plain_indices,
+             &src_layout]() {
+              auto batch_count =
+                  resultBuffer.dim() > 2 ? resultBuffer.size(-3) : 1;
               auto values_accessor = valuesBuffer.accessor<scalar_t, 2>();
               scalar_t* out_ptr = resultBuffer.data_ptr<scalar_t>();
               scalar_t cast_value = alpha.to<scalar_t>();
 
-              auto crow_indices_accessor =
-                  src_crow_indices.accessor<index_t, 2>();
-              auto col_indices_accessor =
-                  src_col_indices.accessor<index_t, 2>();
+              auto compressed_indices_accessor =
+                  src_compressed_indices.accessor<index_t, 2>();
+              auto plain_indices_accessor =
+                  src_plain_indices.accessor<index_t, 2>();
               auto out_strides = resultBuffer.strides();
+              auto const out_stride_batch = out_strides[0];
+              auto const out_stride_compressed =
+                  AT_DISPATCH_ROW_SPARSE_COMPRESSED_LAYOUTS(
+                      src_layout,
+                      "add_out_dense_sparse_compressed_cpu",
+                      [&out_strides] { return out_strides[1]; },
+                      [&out_strides] { return out_strides[2]; });
+              auto const out_stride_plain =
+                  AT_DISPATCH_ROW_SPARSE_COMPRESSED_LAYOUTS(
+                      src_layout,
+                      "add_out_dense_sparse_compressed_cpu",
+                      [&out_strides] { return out_strides[2]; },
+                      [&out_strides] { return out_strides[1]; });
 
               for (const auto batch_idx : c10::irange(batch_count)) {
-                for (const auto irow : c10::irange(src_crow_indices.size(-1) - 1)) {
-                  index_t start_index = crow_indices_accessor[batch_idx][irow];
-                  index_t end_index = crow_indices_accessor[batch_idx][irow + 1];
+                for (const auto i_compressed :
+                     c10::irange(src_compressed_indices.size(-1) - 1)) {
+                  index_t start_index =
+                      compressed_indices_accessor[batch_idx][i_compressed];
+                  index_t end_index =
+                      compressed_indices_accessor[batch_idx][i_compressed + 1];
                   for (const auto i : c10::irange(start_index, end_index)) {
-                    auto icol = col_indices_accessor[batch_idx][i];
-                    auto index = batch_idx * out_strides[0] + irow * out_strides[1] + icol * out_strides[2];
-                    out_ptr[index] += cast_value * values_accessor[batch_idx][i];
+                    auto i_plain = plain_indices_accessor[batch_idx][i];
+                    auto index = batch_idx * out_stride_batch +
+                        i_compressed * out_stride_compressed +
+                        i_plain * out_stride_plain;
+                    out_ptr[index] +=
+                        cast_value * values_accessor[batch_idx][i];
                   }
                 }
               }
@@ -951,13 +957,15 @@ static void add_out_dense_sparse_csr_cpu(
   }
 }
 
-Tensor& add_out_sparse_csr_cpu(
+Tensor& add_out_sparse_compressed_cpu(
     const Tensor& self,
     const SparseCsrTensor& other,
     const Scalar& alpha,
     SparseCsrTensor& out) {
   if (self.layout() == kStrided) {
-    add_out_dense_sparse_csr_cpu(out, self, other, alpha);
+    add_out_dense_sparse_compressed_cpu(out, self, other, alpha);
+  } else if (other.layout() == kStrided) {
+    add_out_dense_sparse_compressed_cpu(out, other, self, alpha);
   } else {
     TORCH_CHECK(
         self.sizes().equals(other.sizes()),
@@ -992,7 +1000,7 @@ struct Reduction...Op {
   inline scalar_t identity() const { return ...; }
 };
 
-Tensor _sparse_csr_..._cpu(const Tensor& input, IntArrayRef dims_to_sum, bool keepdim, c10::optional<ScalarType> dtype) {
+Tensor _sparse_csr_..._cpu(const Tensor& input, IntArrayRef dims_to_sum, bool keepdim, std::optional<ScalarType> dtype) {
   ...
       result = reduce_sparse_csr_cpu_template<scalar_t>(input_, dims_to_sum, keepdim, Reduction...Op<scalar_t>());
   ...
@@ -1056,8 +1064,6 @@ Tensor reduce_sparse_csr_dim0_cpu_template(const Tensor& sparse, ReductionOp rop
   Tensor col_indices = sparse.col_indices();
   Tensor values = sparse.values();
   auto numel = values.numel();
-  Tensor new_col_indices;
-  Tensor columns_map;
 
   /*
     Calling at::_unique constitutes the main bottleneck of this
@@ -1065,7 +1071,7 @@ Tensor reduce_sparse_csr_dim0_cpu_template(const Tensor& sparse, ReductionOp rop
     invariant:
       csr.sum(dim=0) == csr.transpose(0, 1).sum(dim=1)
   */
-  std::tie(new_col_indices, columns_map) = at::_unique(col_indices, true, true);
+  auto [new_col_indices, columns_map] = at::_unique(col_indices, true, true);
   auto nnz = new_col_indices.numel();
 
   Tensor new_crow_indices = at::empty({2}, col_indices.options());
@@ -1330,7 +1336,7 @@ struct ReductionMulOp {
 
 }  // namespace
 
-Tensor _sparse_csr_sum_cpu(const Tensor& input, IntArrayRef dims_to_sum, bool keepdim, c10::optional<ScalarType> dtype) {
+Tensor _sparse_csr_sum_cpu(const Tensor& input, IntArrayRef dims_to_sum, bool keepdim, std::optional<ScalarType> dtype) {
   ScalarType dtype_ = dtype.value_or(input.scalar_type());
   Tensor input_ = at::sparse_csr::to_type(input, dtype_);
   Tensor result;
@@ -1346,7 +1352,7 @@ Tensor _sparse_csr_sum_cpu(const Tensor& input, IntArrayRef dims_to_sum, bool ke
   return result;
 }
 
-Tensor _sparse_csr_prod_cpu(const Tensor& input, IntArrayRef dims_to_reduce, bool keepdim, c10::optional<ScalarType> dtype) {
+Tensor _sparse_csr_prod_cpu(const Tensor& input, IntArrayRef dims_to_reduce, bool keepdim, std::optional<ScalarType> dtype) {
   ScalarType dtype_ = dtype.value_or(input.scalar_type());
   Tensor input_ = input.to(dtype_);
   Tensor result;
@@ -1446,7 +1452,7 @@ std::tuple<Tensor, Tensor> _sparse_mm_reduce_impl_backward_sparse_csr_cpu(
         /*transpose*/false);
     row = coo_indices.select(0, 0);
 
-    // calculte the global index for CSC
+    // calculate the global index for CSC
     // and get the conversion permute pattern
     Tensor index = col.mul(self.size(0)).add_(row);
     permute = index.argsort();
