@@ -8,7 +8,7 @@ import torch
 import torch.distributed as dist
 import torch.testing._internal.common_methods_invocations as common_ops
 
-from torch.distributed._tensor import DeviceMesh, DTensor, Replicate
+from torch.distributed._tensor import DeviceMesh, DTensor
 
 from torch.overrides import resolve_name
 from torch.testing._internal.common_device_type import (
@@ -25,7 +25,8 @@ from torch.testing._internal.distributed._tensor.common_dtensor import (
     DTensorConverter,
     DTensorOpTestBase,
 )
-from torch.utils._pytree import tree_flatten, tree_map
+from torch.utils import _pytree as pytree
+from torch.utils._pytree import tree_map
 
 # rewrite common size variables to sth can be sharded evenly
 # we can enable uneven shards later, but need to adjust more on
@@ -93,8 +94,8 @@ dtensor_fails = {
     # get full support with varying sharding specs
     xfail("__getitem__"),
     xfail("__rsub__"),
+    xfail("_chunk_cat"),
     xfail("_native_batch_norm_legit"),
-    xfail("_softmax_backward_data"),
     xfail("_upsample_bilinear2d_aa"),
     xfail("addbmm"),
     xfail("addmv"),
@@ -112,12 +113,11 @@ dtensor_fails = {
     xfail("as_strided"),
     xfail("as_strided", "partial_views"),
     xfail("as_strided_scatter"),
-    xfail("baddbmm"),
     xfail("bernoulli"),
+    xfail("_batch_norm_with_update"),
     xfail("block_diag"),
     xfail("broadcast_shapes"),
     xfail("cauchy"),
-    xfail("cartesian_prod"),
     xfail("cdist"),
     xfail("cholesky"),
     xfail("cholesky_inverse"),
@@ -131,7 +131,6 @@ dtensor_fails = {
     xfail("constant_pad_nd"),
     xfail("corrcoef"),
     xfail("count_nonzero"),
-    xfail("cov"),
     xfail("cross"),
     xfail("cummax"),
     xfail("cummin"),
@@ -147,9 +146,11 @@ dtensor_fails = {
     xfail("dot"),
     xfail("einsum"),
     xfail("empty"),
+    xfail("empty_strided"),
     xfail("empty_like"),
     xfail("empty_permuted"),
     xfail("exponential"),
+    xfail("equal"),
     xfail("eye"),
     xfail("fft.fft2"),
     xfail("fft.fft"),
@@ -189,7 +190,10 @@ dtensor_fails = {
     xfail("index_copy"),
     xfail("index_fill"),
     xfail("index_put"),
-    xfail("index_reduce"),
+    xfail("index_reduce", "prod"),
+    xfail("index_reduce", "mean"),
+    xfail("index_reduce", "amax"),
+    xfail("index_reduce", "amin"),
     xfail("index_select"),
     xfail("isin"),
     xfail("isinf"),
@@ -198,7 +202,6 @@ dtensor_fails = {
     xfail("kthvalue"),
     xfail("linalg.cholesky"),
     xfail("linalg.cholesky_ex"),
-    xfail("linalg.cond"),
     xfail("linalg.cross"),
     xfail("linalg.det"),
     xfail("linalg.det", "singular"),
@@ -233,20 +236,18 @@ dtensor_fails = {
     xfail("linalg.solve"),
     xfail("linalg.solve_ex"),
     xfail("linalg.solve_triangular"),
-    xfail("linalg.svd"),
-    xfail("linalg.svdvals"),
     xfail("linalg.tensorinv"),
     xfail("linalg.tensorsolve"),
     xfail("linalg.vander"),
     xfail("linalg.vecdot"),
-    xfail("linalg.vector_norm"),
     xfail("linspace"),
+    xfail("linspace", "tensor_overload"),
     xfail("log_normal"),
     xfail("logcumsumexp"),
     xfail("logdet"),
     xfail("logspace"),
+    xfail("logspace", "tensor_overload"),
     xfail("logsumexp"),
-    xfail("lt"),
     xfail("lu"),
     xfail("lu_solve"),
     xfail("lu_unpack"),
@@ -259,23 +260,14 @@ dtensor_fails = {
     xfail("masked.argmin"),
     xfail("masked.cumprod"),
     xfail("masked.cumsum"),
-    xfail("masked.log_softmax"),
-    xfail("masked.logaddexp"),
     xfail("masked.logsumexp"),
     xfail("masked.median"),
-    xfail("masked.norm"),
-    xfail("masked.prod"),
-    xfail("masked.softmin"),
-    xfail("masked.softmax"),
-    xfail("masked.sum"),
     xfail("matrix_exp"),
     xfail("max", "binary"),
-    xfail("max", "reduction_no_dim"),
     xfail("max", "reduction_with_dim"),
     xfail("maximum"),
     xfail("median"),
     xfail("min", "binary"),
-    xfail("min", "reduction_no_dim"),
     xfail("min", "reduction_with_dim"),
     xfail("minimum"),
     xfail("mode"),
@@ -289,7 +281,6 @@ dtensor_fails = {
     xfail("nansum"),
     xfail("native_batch_norm"),
     xfail("native_dropout_backward"),
-    xfail("native_layer_norm"),
     xfail("narrow_copy"),
     xfail("ne"),
     xfail("new_empty"),
@@ -313,11 +304,11 @@ dtensor_fails = {
     xfail("nn.functional.celu"),
     xfail("nn.functional.conv1d"),
     xfail("nn.functional.conv2d"),
+    xfail("nn.functional.conv3d"),
     xfail("nn.functional.conv_transpose1d"),
     xfail("nn.functional.conv_transpose2d"),
     xfail("nn.functional.conv_transpose3d"),
     xfail("nn.functional.cosine_similarity"),
-    xfail("nn.functional.cross_entropy"),
     xfail("nn.functional.ctc_loss"),
     xfail("nn.functional.dropout"),
     xfail("nn.functional.dropout2d"),
@@ -340,8 +331,8 @@ dtensor_fails = {
     xfail("nn.functional.interpolate", "bilinear"),
     xfail("nn.functional.interpolate", "linear"),
     xfail("nn.functional.interpolate", "nearest"),
+    xfail("nn.functional.interpolate", "nearest-exact"),
     xfail("nn.functional.interpolate", "trilinear"),
-    xfail("nn.functional.layer_norm"),
     xfail("nn.functional.leaky_relu"),
     xfail("nn.functional.linear"),
     xfail("nn.functional.local_response_norm"),
@@ -359,14 +350,14 @@ dtensor_fails = {
     xfail("nn.functional.mish"),
     xfail("nn.functional.mse_loss"),
     xfail("nn.functional.multi_margin_loss"),
+    xfail("nn.functional.multi_head_attention_forward"),
     xfail("nn.functional.multilabel_margin_loss"),
     xfail("nn.functional.multilabel_soft_margin_loss"),
-    xfail("nn.functional.nll_loss"),
     xfail("nn.functional.normalize"),
-    xfail("nn.functional.pad", "circular"),
     xfail("nn.functional.pad", "constant"),
     xfail("nn.functional.pad", "reflect"),
     xfail("nn.functional.pad", "replicate"),
+    xfail("nn.functional.pad", "replicate_negative"),
     xfail("nn.functional.pairwise_distance"),
     xfail("nn.functional.pdist"),
     xfail("nn.functional.pixel_shuffle"),
@@ -376,7 +367,6 @@ dtensor_fails = {
     xfail("nn.functional.relu6"),
     xfail("nn.functional.rrelu"),
     xfail("nn.functional.selu"),
-    xfail("nn.functional.silu"),
     xfail("nn.functional.smooth_l1_loss"),
     xfail("nn.functional.soft_margin_loss"),
     xfail("nn.functional.softplus"),
@@ -388,10 +378,6 @@ dtensor_fails = {
     xfail("nn.functional.upsample_bilinear"),
     xfail("nn.functional.upsample_nearest"),
     xfail("nonzero"),
-    xfail("norm"),
-    xfail("norm", "fro"),
-    xfail("norm", "inf"),
-    xfail("norm", "nuc"),
     xfail("normal"),
     xfail("normal", "number_mean"),
     xfail("normal", "in_place"),
@@ -417,7 +403,6 @@ dtensor_fails = {
     xfail("rsub"),
     xfail("scalar_tensor"),
     xfail("scatter_add"),
-    xfail("scatter"),
     xfail("scatter_reduce", "amax"),
     xfail("scatter_reduce", "amin"),
     xfail("scatter_reduce", "mean"),
@@ -473,14 +458,10 @@ dtensor_fails = {
     xfail("std_mean"),
     xfail("std_mean", "unbiased"),
     xfail("stft"),
-    xfail("svd"),
     xfail("svd_lowrank"),
-    xfail("t"),
-    xfail("take_along_dim"),
     xfail("take"),
     xfail("tensor_split"),
     xfail("to_sparse"),
-    xfail("topk"),
     xfail("trace"),
     xfail("trapezoid"),
     xfail("trapz"),
@@ -495,16 +476,15 @@ dtensor_fails = {
     xfail("unique_consecutive"),
     xfail("unique"),
     xfail("unsafe_split"),
+    xfail("unsafe_chunk"),
     xfail("var_mean"),
     xfail("var_mean", "unbiased"),
     xfail("vdot"),
     xfail("view_copy"),
-    xfail("view_as_complex"),
-    xfail("where"),
     xfail("zeros"),
     # ops inside this might even fail without dtensor
     # tests, as we rescale op db common test size factor (i.e. L, M, S)
-    # which triggered the orignal function run failures with input
+    # which triggered the original function run failures with input
     # generation becomes wrong, we skip them for now but should enable later.
     # TODO: need to clean this list and remove all cases
     skip("argwhere"),
@@ -590,8 +570,8 @@ class TestDTensorOps(DTensorOpTestBase):
         self.check_dtensor_func(test, op)
 
     def assert_ref_dtensor_equal(self, dtensor_rs, rs):
-        flat_dtensor_rs, _ = tree_flatten(dtensor_rs)
-        flat_rs, _ = tree_flatten(rs)
+        flat_dtensor_rs = pytree.tree_leaves(dtensor_rs)
+        flat_rs = pytree.tree_leaves(rs)
         self.assertEqual(len(flat_dtensor_rs), len(flat_rs))
         for dtensor_r, r in zip(flat_dtensor_rs, flat_rs):
             if not isinstance(r, torch.Tensor):
@@ -611,7 +591,7 @@ class TestDTensorOps(DTensorOpTestBase):
                 f"dtensor requires_grad: {dtensor_r.requires_grad}",
             )
 
-            self.assertEqualOnRank(dtensor_r.to_local(), r)
+            self.assertEqualOnRank(dtensor_r, r)
 
     def run_dtensor_crossref(self, func, args, kwargs):
         to_dtensor = DTensorConverter(self.mesh, args, kwargs)
@@ -630,11 +610,7 @@ class TestDTensorOps(DTensorOpTestBase):
         rs = concat_res_if_necessary(func, rs)
 
         def to_replicate(e: object) -> object:
-            return (
-                e.redistribute(self.mesh, self.mesh.ndim * [Replicate()])
-                if isinstance(e, DTensor)
-                else e
-            )
+            return e.full_tensor() if isinstance(e, DTensor) else e
 
         try:
             # Suppress warnings, this doesn't matter for test_meta.py
@@ -656,10 +632,10 @@ class TestDTensorOps(DTensorOpTestBase):
                         # errors
                         dtensor_rs = func(*dtensor_args, **dtensor_kwargs)
 
-                        # we need to skip tests containing tensors of zero elmeents for now.
+                        # we need to skip tests containing tensors of zero elements for now.
                         # see issue: https://github.com/pytorch/tau/issues/470
                         # TODO remove this once issue above fixed.
-                        flat_args, _ = tree_flatten(dtensor_rs)
+                        flat_args = pytree.tree_leaves(dtensor_rs)
                         if any(
                             isinstance(e, torch.Tensor) and e.numel() == 0
                             for e in flat_args
@@ -713,7 +689,4 @@ instantiate_device_type_tests(TestDTensorOps, globals(), only_for=(DEVICE_TYPE,)
 
 
 if __name__ == "__main__":
-    # NB: CPU dtensor ops test frequently timeout https://github.com/pytorch/pytorch/issues/98816
-    # so running it only on CUDA
-    if torch.cuda.is_available():
-        run_tests()
+    run_tests()
