@@ -1,26 +1,29 @@
 # Owner(s): ["oncall: jit"]
 
+import gc
 import os
 import sys
-import gc
 import unittest
+from typing import NamedTuple
 
 import torch
-from typing import NamedTuple
 from torch.testing import FileCheck
+from torch.testing._internal.common_cuda import TEST_MULTIGPU
+from torch.testing._internal.common_utils import (
+    NoTest,
+    skipCUDANonDefaultStreamIf,
+    skipIfRocm,
+    TEST_CUDA,
+)
 from torch.testing._internal.jit_utils import JitTestCase
-from torch.testing._internal.common_utils import skipIfRocm, skipCUDANonDefaultStreamIf, NoTest, TEST_CUDA
 
 # Make the helper files in test/ importable
 pytorch_test_dir = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
 sys.path.append(pytorch_test_dir)
 
-# Check if multiple GPU's are available
-TEST_MULTIGPU = TEST_CUDA and torch.cuda.device_count() >= 2
-
 # If GPU is not available, then do not run the tests
 if not TEST_CUDA:
-    print('CUDA not available, skipping tests', file=sys.stderr)
+    print("CUDA not available, skipping tests", file=sys.stderr)
     JitTestCase = NoTest  # noqa: F811
 
 TEST_LARGE_TENSOR = TEST_CUDA
@@ -38,10 +41,12 @@ if __name__ == "__main__":
         "instead."
     )
 
+
 class TestCUDA(JitTestCase):
     """
     A suite of tests for the CUDA API in TorchScript.
     """
+
     def tearDown(self):
         gc.collect()
         torch.cuda.empty_cache()
@@ -56,10 +61,10 @@ class TestCUDA(JitTestCase):
         def test_device_synchronize():
             prev_current_device_index = torch.cuda.current_device()
             torch.cuda.synchronize()
-            torch.cuda.synchronize('cuda')
-            torch.cuda.synchronize('cuda:0')
+            torch.cuda.synchronize("cuda")
+            torch.cuda.synchronize("cuda:0")
             torch.cuda.synchronize(0)
-            torch.cuda.synchronize(torch.device('cuda:1'))
+            torch.cuda.synchronize(torch.device("cuda:1"))
             after_current_device_index = torch.cuda.current_device()
 
             # Check if the current device index is same as the device index before
@@ -68,7 +73,7 @@ class TestCUDA(JitTestCase):
 
         @torch.jit.script
         def test_multi_device_synchronize():
-            torch.cuda.synchronize(torch.device('cuda:0'))
+            torch.cuda.synchronize(torch.device("cuda:0"))
             prev_current_device_index = torch.cuda.current_device()
             torch.cuda.synchronize(1)
             after_current_device_index = torch.cuda.current_device()
@@ -78,11 +83,9 @@ class TestCUDA(JitTestCase):
             return prev_current_device_index == after_current_device_index
 
         self.assertTrue(test_device_synchronize)
-        FileCheck().check("cuda::synchronize(") \
-                   .run(test_device_synchronize.graph)
+        FileCheck().check("cuda::synchronize(").run(test_device_synchronize.graph)
         self.assertTrue(test_multi_device_synchronize)
-        FileCheck().check("cuda::synchronize(") \
-                   .run(test_multi_device_synchronize.graph)
+        FileCheck().check("cuda::synchronize(").run(test_multi_device_synchronize.graph)
 
     def test_stream_args(self):
         # Test stream creation with default arguments
@@ -167,7 +170,6 @@ class TestCUDA(JitTestCase):
     @unittest.skipIf(not TEST_LARGE_TENSOR, "not enough memory")
     @skipCUDANonDefaultStreamIf(True)
     def test_streams_and_events(self):
-
         # Test default_stream API by passing device ID as an argument and
         # and check if the stream device index matches with the device ID
         @torch.jit.script
@@ -184,14 +186,14 @@ class TestCUDA(JitTestCase):
         # This test checks for the default stream ID is set to 0 on the device
         @torch.jit.script
         def test_default_streams():
-            s0 = torch.cuda.default_stream(torch.device('cuda:0'))
-            s1 = torch.cuda.default_stream(torch.device('cuda:1'))
+            s0 = torch.cuda.default_stream(torch.device("cuda:0"))
+            s1 = torch.cuda.default_stream(torch.device("cuda:1"))
 
-            d = torch.device('cuda:1')
+            d = torch.device("cuda:1")
 
             # Check the current stream id and default id are same
             # on the current device. The current device id by default is 0
-            s2 = torch.cuda.current_stream(torch.device('cuda:0'))
+            s2 = torch.cuda.current_stream(torch.device("cuda:0"))
             check_s2 = s2.id() == s0.id()
             check_d0 = torch.cuda.current_device() == s2.device_index()
 
@@ -205,9 +207,25 @@ class TestCUDA(JitTestCase):
             # Check if the current device was reset to 0
             is_device_d0 = torch.cuda.current_device() == s2.device_index()
 
-            return s0.device_index(), s1.device_index(), check_s2, check_s3, check_d0, check_d1, is_device_d0
+            return (
+                s0.device_index(),
+                s1.device_index(),
+                check_s2,
+                check_s3,
+                check_d0,
+                check_d1,
+                is_device_d0,
+            )
 
-        d0, d1, check_s2, check_s3, check_d0, check_d1, is_device_d0 = test_default_streams()
+        (
+            d0,
+            d1,
+            check_s2,
+            check_s3,
+            check_d0,
+            check_d1,
+            is_device_d0,
+        ) = test_default_streams()
 
         self.assertEqual(d0, 0)
         self.assertEqual(d1, 1)
@@ -230,12 +248,21 @@ class TestCUDA(JitTestCase):
             with torch.cuda.stream(None):
                 cur_device_index = torch.cuda.current_device()
                 is_device_index_same = cur_device_index == device_index
-                is_current_stream_same = torch.cuda.current_stream(device).id() == current_stream.id()
-                is_default_stream_same = torch.cuda.default_stream(device).id() == default_stream.id()
+                is_current_stream_same = (
+                    torch.cuda.current_stream(device).id() == current_stream.id()
+                )
+                is_default_stream_same = (
+                    torch.cuda.default_stream(device).id() == default_stream.id()
+                )
 
             # Check if the device index, current stream and default streams have not changed
-            are_streams_same = is_device_index_same and is_current_stream_same and is_default_stream_same
+            are_streams_same = (
+                is_device_index_same
+                and is_current_stream_same
+                and is_default_stream_same
+            )
             return are_streams_same
+
         self.assertTrue(test_set_none_stream())
 
         # This test checks if the Device Context manager is a no op
@@ -248,6 +275,7 @@ class TestCUDA(JitTestCase):
                 # Check if the current device is the same
                 is_device_same = torch.cuda.current_device() == device_index
             return is_device_same
+
         self.assertTrue(test_set_device_none())
 
         # Check if a CUDA JIT stream is created
@@ -262,15 +290,15 @@ class TestCUDA(JitTestCase):
 
         # Class used to store results for the test: test_get_stream.
         class Result(NamedTuple):
-            t1 : torch.Tensor
-            t2 : torch.Tensor
-            is_current_and_default_stream_same : bool
-            is_default_and_user_stream_not_same : bool
-            is_stream_set : bool
-            is_stream_reset : bool
-            default_stream_query : bool
-            default_stream_id : int
-            user_stream_id : int
+            t1: torch.Tensor
+            t2: torch.Tensor
+            is_current_and_default_stream_same: bool
+            is_default_and_user_stream_not_same: bool
+            is_stream_set: bool
+            is_stream_reset: bool
+            default_stream_query: bool
+            default_stream_id: int
+            user_stream_id: int
 
         # The test aims at checking different stream proporties.
         @torch.jit.script
@@ -282,15 +310,23 @@ class TestCUDA(JitTestCase):
             user_stream = torch.cuda.Stream()
 
             # Check if the current and default streams are the same on the device
-            is_current_and_default_stream_same = current_stream.id() == default_stream.id()
+            is_current_and_default_stream_same = (
+                current_stream.id() == default_stream.id()
+            )
             # Check if user stream and default stream are not the same on the device
-            is_default_and_user_stream_not_same = default_stream.id() != user_stream.id()
+            is_default_and_user_stream_not_same = (
+                default_stream.id() != user_stream.id()
+            )
 
             with torch.cuda.stream(user_stream):
-                is_stream_set = torch.cuda.current_stream(device).id() == user_stream.id()
+                is_stream_set = (
+                    torch.cuda.current_stream(device).id() == user_stream.id()
+                )
 
             # Check if the stream was reset to current_stream
-            is_stream_reset = torch.cuda.current_stream(device).id() == current_stream.id()
+            is_stream_reset = (
+                torch.cuda.current_stream(device).id() == current_stream.id()
+            )
 
             tensor1 = torch.rand(10000, 10000, device="cuda")
             tensor2 = torch.mm(tensor1, tensor1).to("cuda")
@@ -299,9 +335,16 @@ class TestCUDA(JitTestCase):
 
             # Capture all the results in the class Result
             res = Result(
-                tensor1, tensor2, is_current_and_default_stream_same,
-                is_default_and_user_stream_not_same, is_stream_set,
-                is_stream_reset, default_stream_query, default_stream.id(), user_stream.id())
+                tensor1,
+                tensor2,
+                is_current_and_default_stream_same,
+                is_default_and_user_stream_not_same,
+                is_stream_set,
+                is_stream_reset,
+                default_stream_query,
+                default_stream.id(),
+                user_stream.id(),
+            )
             return res
 
         result = test_get_stream()
@@ -312,8 +355,12 @@ class TestCUDA(JitTestCase):
         self.assertTrue(result.is_stream_set)
         self.assertTrue(result.is_stream_reset)
         self.assertTrue(result.default_stream_query)
-        self.assertEqual(result.default_stream_id, 0)  # Check if the default stream ID is always 0
-        self.assertNotEqual(result.user_stream_id, 0)  # Check if the user stream is always non zero
+        self.assertEqual(
+            result.default_stream_id, 0
+        )  # Check if the default stream ID is always 0
+        self.assertNotEqual(
+            result.user_stream_id, 0
+        )  # Check if the user stream is always non zero
 
         # Test the stream context manager. This test checks if the stream is switched
         # to the user stream on using the stream context manager.
@@ -331,14 +378,20 @@ class TestCUDA(JitTestCase):
             # Wait for B to be computed
             user_stream.synchronize()
             # Check if the stream has been reset on the current device
-            is_stream_reset = torch.cuda.current_stream(device).id() == current_stream.id()
+            is_stream_reset = (
+                torch.cuda.current_stream(device).id() == current_stream.id()
+            )
 
             return A, B, check, is_stream_reset
 
         A, B, is_stream_set, is_stream_reset = test_stream_context()
         self.assertEqual(torch.matmul(A, A), B)
-        self.assertTrue(is_stream_set, "Error: Current stream was not set to user stream!")
-        self.assertTrue(is_stream_reset, "Error: The stream was not restored to previous stream!")
+        self.assertTrue(
+            is_stream_set, "Error: Current stream was not set to user stream!"
+        )
+        self.assertTrue(
+            is_stream_reset, "Error: The stream was not restored to previous stream!"
+        )
 
         # Test multiple nested streams. Check if the operations are computed as expected on the streams
         # This test has been adapted from the eager mode tests available at test/test_cuda.py
@@ -374,11 +427,24 @@ class TestCUDA(JitTestCase):
 
             # Check if the stream and device has been restored to previous stream and device
             is_device_current = torch.cuda.current_device() == prev_device_index
-            is_stream_current = torch.cuda.current_stream(device).id() == prev_current_stream.id()
+            is_stream_current = (
+                torch.cuda.current_stream(device).id() == prev_current_stream.id()
+            )
 
-            check_stream = is_stream_s1 and is_stream_s2 and is_stream_s1_after and is_stream_current
-            check_device = is_device_s1 and is_device_s2 and is_device_s1_after and is_device_current
+            check_stream = (
+                is_stream_s1
+                and is_stream_s2
+                and is_stream_s1_after
+                and is_stream_current
+            )
+            check_device = (
+                is_device_s1
+                and is_device_s2
+                and is_device_s1_after
+                and is_device_current
+            )
             return A, B, C, D, check_stream, check_device
+
         A, B, C, D, check_stream, check_device = test_multiple_stream()
 
         self.assertEqual(torch.matmul(A, A), C)
@@ -403,7 +469,9 @@ class TestCUDA(JitTestCase):
                 B = torch.mm(A, A).to("cuda")
             s1.record_event(event)
             # Check if the current_stream is reset
-            is_current_stream_1 = torch.cuda.current_stream(device).id() == prev_current_stream.id()
+            is_current_stream_1 = (
+                torch.cuda.current_stream(device).id() == prev_current_stream.id()
+            )
             # Wait for ops on s1 to be computed
             s2.wait_event(event)
             with torch.cuda.stream(s2):
@@ -412,9 +480,16 @@ class TestCUDA(JitTestCase):
             # Wait for C to be computed
             s2.synchronize()
             # Check if the current_stream is reset
-            is_current_stream_2 = torch.cuda.current_stream(device).id() == prev_current_stream.id()
+            is_current_stream_2 = (
+                torch.cuda.current_stream(device).id() == prev_current_stream.id()
+            )
 
-            check_stream = is_current_stream_1 and is_current_stream_2 and is_stream_s1 and is_stream_s2
+            check_stream = (
+                is_current_stream_1
+                and is_current_stream_2
+                and is_stream_s1
+                and is_stream_s2
+            )
             return A, B, C, check_stream
 
         A, B, C, check_stream = test_data_dependency_between_streams()
@@ -427,6 +502,7 @@ class TestCUDA(JitTestCase):
         def test_simple_event():
             e = torch.cuda.Event(True, False, False)
             return e is not None
+
         self.assertTrue(test_simple_event(), "Could not create CUDA Event!")
 
         # Record the CUDA event for operation torch.mm on the current stream
@@ -476,6 +552,7 @@ class TestCUDA(JitTestCase):
             # not necessary to check e_tik and e_tok, as elapsed_time would throw
             # exception if otherwise.
             return e_tik.elapsed_time(e_tok)
+
         self.assertGreater(test_stream_synchronize(), 0)
 
         # Test event synchronization for the event that records a stream doing
@@ -538,12 +615,13 @@ class TestCUDA(JitTestCase):
             # not necessary to check e_tik and e_tok, as elapsed_time would throw
             # exception if otherwise.
             return e_tik.elapsed_time(e_tok)
+
         self.assertGreater(test_event_wait(), 0)
 
         # Test for stream wait_event. Checks if the stream waits on the event
         @torch.jit.script
         def test_wait_event():
-            d1 = torch.device('cuda:1')
+            d1 = torch.device("cuda:1")
 
             with torch.cuda.device(d1):
                 s0 = torch.cuda.current_stream(d1)
@@ -552,11 +630,12 @@ class TestCUDA(JitTestCase):
                 e0 = torch.cuda.Event(False, False, False)
                 s0.record_event(e0)
 
-            s1 = torch.cuda.current_stream(torch.device('cuda:0'))
+            s1 = torch.cuda.current_stream(torch.device("cuda:0"))
             s1.wait_event(e0)
             s1.synchronize()
 
             return e0.query() and s0.query() and s1.query()
+
         self.assertTrue(test_wait_event())
 
         # Test if a scripted module with cuda streams can be saved, loaded and executed
