@@ -15,7 +15,7 @@ from torch.testing._internal.common_device_type import (
 from torch.testing._internal.common_modules import module_db, modules, ModuleErrorEnum, TrainEvalMode
 from torch.testing._internal.common_utils import (
     TestCase, run_tests, freeze_rng_state, mock_wrapper, get_tensors_from, gradcheck,
-    gradgradcheck)
+    gradgradcheck, parametrize, wrapSwapTensorsTest)
 from unittest.mock import patch, call
 
 
@@ -212,7 +212,7 @@ class TestModule(TestCase):
             str(m)
 
     @modules(module_db)
-    def test_pickle(self, device, dtype, module_info, training):
+    def test_save_load(self, device, dtype, module_info, training):
         # Test that module can be pickled and unpickled.
         module_cls = module_info.module_cls
         module_inputs = module_info.module_inputs_func(module_info, device=device, dtype=dtype,
@@ -229,12 +229,13 @@ class TestModule(TestCase):
                 m = module_cls(*args, **kwargs)
                 m.to(device).to(dtype)
                 m.train(training)
+                sd = m.state_dict()
 
                 # === Do forward pass. ===
                 args, kwargs = module_input.forward_input.args, module_input.forward_input.kwargs
                 output = m(*args, **kwargs)
 
-                # === Check unpickled module gives the same output. ===
+                # === Check saved/loaded module gives the same output. ===
                 with tempfile.TemporaryFile() as f:
                     torch.save(m, f)
                     f.seek(0)
@@ -242,6 +243,17 @@ class TestModule(TestCase):
                     output_from_copy = m_copy(*args, **kwargs)
                     self.assertEqual(output, output_from_copy)
 
+                # === Check saved/loaded state_dict are the same (including weights_only load). ===
+                with tempfile.TemporaryFile() as f:
+                    torch.save(sd, f)
+                    f.seek(0)
+                    sd_copy = torch.load(f)
+                    self.assertEqual(sd_copy, sd)
+                    del sd_copy
+                    f.seek(0)
+                    sd_copy_wo = torch.load(f, weights_only=True)
+                    self.assertEqual(sd_copy_wo, sd)
+
     @skipMeta
     @modules([module_info for module_info in module_db
               if 'inplace' in signature(module_info.module_cls).parameters])
@@ -304,6 +316,8 @@ class TestModule(TestCase):
             return {name: self._traverse_obj(o, func) for name, o in obj.items()}
         elif isinstance(obj, (torch.Tensor, torch.nn.Parameter)):
             return func(obj)
+        else:
+            return obj
 
     def _retain_grad(self, obj):
         # gradients needs to be retained to check for grad. This is useful when
@@ -380,8 +394,8 @@ class TestModule(TestCase):
                 else:
                     grad_output = tuple(self._traverse_obj(o, lambda o: o.clone().detach_().normal_() if o.requires_grad else None)
                                         for o in default_output)
-                    flattened_default_output, _ = torch.utils._pytree.tree_flatten(default_output)
-                    flattened_grad_output, _ = torch.utils._pytree.tree_flatten(grad_output)
+                    flattened_default_output = torch.utils._pytree.tree_leaves(default_output)
+                    flattened_grad_output = torch.utils._pytree.tree_leaves(grad_output)
                     for o, g_o in zip(flattened_default_output, flattened_grad_output):
                         if (o.requires_grad):
                             o.backward(g_o, retain_graph=True)
@@ -407,8 +421,8 @@ class TestModule(TestCase):
                     if isinstance(out, torch.Tensor):
                         out.backward(g_out_copy, retain_graph=True)
                     else:
-                        flattened_out, _ = torch.utils._pytree.tree_flatten(out)
-                        flattened_g_out_copy, _ = torch.utils._pytree.tree_flatten(g_out_copy)
+                        flattened_out = torch.utils._pytree.tree_leaves(out)
+                        flattened_g_out_copy = torch.utils._pytree.tree_leaves(g_out_copy)
                         for o, g_o in zip(flattened_out, flattened_g_out_copy):
                             if o.requires_grad:
                                 o.backward(g_o, retain_graph=True)
@@ -466,7 +480,7 @@ class TestModule(TestCase):
 
                 with freeze_rng_state():
                     output = m(*new_input_args, **new_kwargs, **other_kwargs)
-                    output_flattened, _ = torch.utils._pytree.tree_flatten(output)
+                    output_flattened = torch.utils._pytree.tree_leaves(output)
                     return output_flattened
 
             # check total derivative
@@ -595,8 +609,8 @@ class TestModule(TestCase):
                 if isinstance(cpu_outputs, torch.Tensor):
                     check_backward(cpu_outputs, gpu_outputs)
                 else:
-                    flatten_cpu_outputs, _ = torch.utils._pytree.tree_flatten(cpu_outputs)
-                    flatten_gpu_outputs, _ = torch.utils._pytree.tree_flatten(gpu_outputs)
+                    flatten_cpu_outputs = torch.utils._pytree.tree_leaves(cpu_outputs)
+                    flatten_gpu_outputs = torch.utils._pytree.tree_leaves(gpu_outputs)
                     for cpu_output, gpu_output in zip(flatten_cpu_outputs, flatten_gpu_outputs):
                         if cpu_output.requires_grad:
                             check_backward(cpu_output, gpu_output)
@@ -649,10 +663,10 @@ class TestModule(TestCase):
                 d = output.dim()
                 if (d == 4 and ((input_mem_format == torch.channels_last)
                                 or (module_mem_format == torch.channels_last and module_memformat_affects_out))):
-                    self.assertTrue(output.is_contiguous(memory_format=torch.channels_last))
+                    self.assertTrue(output.numel() == 0 or output.is_contiguous(memory_format=torch.channels_last))
                 elif (d == 5 and ((input_mem_format == torch.channels_last_3d)
                                   or (module_mem_format == torch.channels_last_3d and module_memformat_affects_out))):
-                    self.assertTrue(output.is_contiguous(memory_format=torch.channels_last_3d))
+                    self.assertTrue(output.numel() == 0 or output.is_contiguous(memory_format=torch.channels_last_3d))
                 else:
                     self.assertTrue(output.is_contiguous())
             return self._traverse_obj(output, inner_check_out_mem_format)
@@ -680,12 +694,12 @@ class TestModule(TestCase):
                 args, kwargs = module_input.forward_input.args, module_input.forward_input.kwargs
                 desired_outputs = m(*args, **kwargs)
                 # === Do backward pass. ===
-                ref_diff_outputs = tuple(t for t in torch.utils._pytree.tree_flatten(desired_outputs)[0] if _req_grad(t))
+                ref_diff_outputs = tuple(t for t in torch.utils._pytree.tree_leaves(desired_outputs) if _req_grad(t))
                 if training and len(ref_diff_outputs) > 0:
                     params = tuple(p for p in m.parameters())
                     ref_diff_inputs = tuple(
                         t
-                        for t in torch.utils._pytree.tree_flatten((args, kwargs, params))[0]
+                        for t in torch.utils._pytree.tree_leaves((args, kwargs, params))
                         if _req_grad(t)
                     )
                     ref_grad_outputs = tuple(
@@ -706,13 +720,13 @@ class TestModule(TestCase):
                     # See https://github.com/pytorch/pytorch/issues/107861
                     # When inductor tests are turned on, the setting of requires_grad will be lost
                     for t1, t2 in zip(
-                        torch.utils._pytree.tree_flatten(d_args)[0],
-                        torch.utils._pytree.tree_flatten(module_input.forward_input.args)[0],
+                        torch.utils._pytree.tree_leaves(d_args),
+                        torch.utils._pytree.tree_leaves(module_input.forward_input.args),
                     ):
                         t1.requires_grad_(t2.requires_grad)
                     for t1, t2 in zip(
-                        torch.utils._pytree.tree_flatten(d_kwargs)[0],
-                        torch.utils._pytree.tree_flatten(module_input.forward_input.kwargs)[0],
+                        torch.utils._pytree.tree_leaves(d_kwargs),
+                        torch.utils._pytree.tree_leaves(module_input.forward_input.kwargs),
                     ):
                         t1.requires_grad_(t2.requires_grad)
 
@@ -735,12 +749,12 @@ class TestModule(TestCase):
                         _check_out_mem_format(outputs, input_mem_format, module_mem_format)
 
                         # === Do backward pass. ===
-                        diff_outputs = tuple(t for t in torch.utils._pytree.tree_flatten(outputs)[0] if _req_grad(t))
+                        diff_outputs = tuple(t for t in torch.utils._pytree.tree_leaves(outputs) if _req_grad(t))
                         if training and len(diff_outputs) > 0:
                             params = tuple(p for p in m.parameters())
                             diff_inputs = tuple(
                                 t
-                                for t in torch.utils._pytree.tree_flatten((args, kwargs, params))[0]
+                                for t in torch.utils._pytree.tree_leaves((args, kwargs, params))
                                 if _req_grad(t)
                             )
                             grad_outputs = tuple(
@@ -814,10 +828,8 @@ class TestModule(TestCase):
 
         for module_input, module_input_meta in zip(module_inputs, module_inputs_meta):
             c_args, c_kwargs = module_input.constructor_input.args, module_input.constructor_input.kwargs
-            fw_args, fw_kwargs = module_input.forward_input.args, module_input.forward_input.kwargs
 
             c_args_meta, c_kwargs_meta = module_input_meta.constructor_input.args, module_input_meta.constructor_input.kwargs
-            fw_args_meta, fw_kwargs_meta = module_input_meta.forward_input.args, module_input_meta.forward_input.kwargs
 
             m_cpu = module_cls(*c_args, **c_kwargs)
 
@@ -851,6 +863,137 @@ class TestModule(TestCase):
             else:
                 raise NotImplementedError(f"Unknown error type {error_input.error_on}")
 
+    # Only run this test for float32 because the test loops over all the dtypes
+    @modules([module for module in module_db if not module.is_lazy], allowed_dtypes=[torch.float32])
+    @parametrize('swap', [True, False])
+    @parametrize('set_grad', [True, False])
+    @wrapSwapTensorsTest()
+    def test_to(self, device, dtype, module_info, training, swap, set_grad):
+        module_cls = module_info.module_cls
+        devices = ['cpu']
+        if torch.cuda.is_available():
+            devices += ['cuda']
+        dtypes = module_info.dtypes
+        module_inputs = module_info.module_inputs_func(module_info, device=device, dtype=dtype,
+                                                       requires_grad=False, training=training)
+        torch.__future__.set_swap_module_params_on_conversion(swap)
+
+        for module_input in module_inputs:
+            c_args, c_kwargs = module_input.constructor_input.args, module_input.constructor_input.kwargs
+            args, kwargs = module_input.forward_input.args, module_input.forward_input.kwargs
+
+            m = module_cls(*c_args, **c_kwargs)
+
+            # Avoid using `module.to()` when constructing module since that is the method we are testing
+            def _to(m, set_grad=False):
+                for c in m.children():
+                    _to(c, set_grad=set_grad)
+                for n, p in m.named_parameters(recurse=False):
+                    new_p = torch.nn.Parameter(p.detach().clone().to(device, dtype))
+                    setattr(m, n, new_p)
+                    if set_grad:
+                        new_p.grad = torch.randn_like(new_p)
+                for n, b in m.named_buffers(recurse=False):
+                    new_b = b.detach().clone().to(device, dtype)
+                    setattr(m, n, new_b)
+            _to(m, set_grad=set_grad)
+
+            # Check .to() can be run after forward and backward with swap
+            has_params = len(list(m.parameters())) > 0
+            if swap and not set_grad and has_params:
+                out = m(*args, **kwargs)
+                if isinstance(out, tuple):
+                    out = out[0]
+                out.sum().backward()
+                m.to(dtype=torch.half)
+                # reset
+                m.to(dtype=torch.float32)
+
+            prev_device, prev_dtype = device, dtype
+            for device_, dtype_ in product(devices, dtypes):
+                # if device/dtype do not change, grad.to(device, dtype) is a no-op so
+                # swapping will not change ._cdata
+                # parameters will be wrapped in an nn.Parameter before swapping
+                # which will cause the ._cdata to change
+                g_no_swap = device_ == prev_device and dtype_ == prev_dtype
+                prev_prev_device, prev_prev_dtype = prev_device, prev_dtype
+                prev_device, prev_dtype = device_, dtype_
+
+                p_ids_before = [id(p) for p in m.parameters()]
+                p_cdatas_before = [p._cdata for p in m.parameters()]
+                if set_grad:
+                    g_ids_before = [id(p.grad) for p in m.parameters()]
+                    g_cdatas_before = [p.grad._cdata for p in m.parameters()]
+
+                m.to(device=device_, dtype=dtype_)
+
+                self.assertTrue(all(isinstance(p, torch.nn.Parameter) for p in m.parameters()))
+                self.assertTrue(all(p.device.type == device_ for p in m.parameters()))
+                self.assertTrue(all(p.dtype == dtype_ for p in m.parameters()))
+                p_ids_after = [id(p) for p in m.parameters()]
+                p_cdatas_after = [p._cdata for p in m.parameters()]
+
+                if set_grad:
+                    self.assertTrue(all(p.grad.device.type == device_ for p in m.parameters()))
+                    self.assertTrue(all(p.grad.dtype == dtype_ for p in m.parameters()))
+                    g_ids_after = [id(p.grad) for p in m.parameters()]
+                    g_cdatas_after = [p.grad._cdata for p in m.parameters()]
+
+                if swap:
+                    # id same, ._cdata differs --> swapped cdata of THPVariable
+                    self.assertTrue(all(a == b for a, b in zip(p_ids_before, p_ids_after)))
+                    self.assertTrue(all(a != b for a, b in zip(p_cdatas_before, p_cdatas_after)))
+                    if set_grad:
+                        self.assertTrue(
+                            all(a == b if g_no_swap else a != b for a, b in zip(g_cdatas_before, g_cdatas_after)))
+                else:
+                    # id and _cdata remain the same --> .data setting
+                    self.assertTrue(all(a == b for a, b in zip(p_cdatas_before, p_cdatas_after)))
+                    self.assertTrue(all(a == b for a, b in zip(p_ids_before, p_ids_after)))
+                    if set_grad:
+                        self.assertTrue(all(a == b for a, b in zip(g_cdatas_before, g_cdatas_after)))
+                        self.assertTrue(all(a == b for a, b in zip(g_ids_before, g_ids_after)))
+
+    @modules([module for module in module_db if not module.is_lazy], allowed_dtypes=[torch.float32])
+    @parametrize('swap', [True, False])
+    @wrapSwapTensorsTest()
+    def test_to_empty(self, device, dtype, module_info, swap, training):
+        module_cls = module_info.module_cls
+
+        with torch.device("meta"):
+            module_inputs = module_info.module_inputs_func(module_info, device=None, dtype=dtype,
+                                                           requires_grad=False, training=training)
+
+        torch.__future__.set_swap_module_params_on_conversion(swap)
+        device_ = torch.device(device)
+
+        for module_input in module_inputs:
+            c_args, c_kwargs = module_input.constructor_input.args, module_input.constructor_input.kwargs
+
+            with torch.device("meta"):
+                m = module_cls(*c_args, **c_kwargs)
+
+            p_ids_before = [id(p) for p in m.parameters()]
+            p_cdatas_before = [p._cdata for p in m.parameters()]
+            m.to_empty(device=device_)
+
+            self.assertTrue(all(isinstance(p, torch.nn.Parameter) for p in m.parameters()))
+            self.assertTrue(all(p.device == device_ for p in m.parameters()))
+            self.assertTrue(all(p.dtype == dtype for p in m.parameters()))
+            p_ids_after = [id(p) for p in m.parameters()]
+            p_cdatas_after = [p._cdata for p in m.parameters()]
+
+            if swap:
+                # id same, ._cdata differs --> swapped cdata of THPVariable
+                self.assertTrue(all(a == b for a, b in zip(p_ids_before, p_ids_after)))
+                self.assertTrue(all(a != b for a, b in zip(p_cdatas_before, p_cdatas_after)))
+            else:
+                # id and ._cdata differ
+                # meta and device have different shallow copy types, so this will create a new
+                # parameter and assign it to the module
+                self.assertTrue(all(a != b for a, b in zip(p_ids_before, p_ids_after)))
+                self.assertTrue(all(a != b for a, b in zip(p_cdatas_before, p_cdatas_after)))
+
 
 instantiate_device_type_tests(TestModule, globals(), allow_mps=True)
 
