@@ -1,3 +1,5 @@
+# mypy: ignore-errors
+
 from typing import Any, Dict, List, Optional, Set, Tuple, Union, Type, Callable
 from torch.ao.quantization.quant_type import QuantType
 import torch
@@ -82,14 +84,25 @@ __all__ = [
     "convert_weighted_module",
 ]
 
+SUPPORTED_QDTYPES = [
+    torch.quint8,
+    torch.qint8,
+    torch.qint32,
+    torch.uint8,
+    torch.int8,
+    torch.int16,
+    torch.int32,
+    torch.float8_e5m2,
+    torch.float8_e4m3fn,
+]
+
 _QSCHEME_TO_CHOOSE_QPARAMS_OP = {
     torch.per_tensor_affine: torch.ops.quantized_decomposed.choose_qparams.tensor,
     torch.per_tensor_symmetric: torch.ops.quantized_decomposed.choose_qparams_symmetric.tensor,
 }
 
 def _replace_observer_with_quantize_dequantize_node_decomposed(
-        model: torch.nn.Module,
-        graph: Graph,
+        model: torch.fx.GraphModule,
         node: Node,
         modules: Dict[str, torch.nn.Module],
         node_name_to_scope: Dict[str, Tuple[str, type]],
@@ -105,10 +118,14 @@ def _replace_observer_with_quantize_dequantize_node_decomposed(
 
     or quantize_per_channel and dequantize_per_channel
     """
+    graph = model.graph
     assert modules is not None
     assert isinstance(node.target, str)
     module_path, prefix = _get_module_path_and_prefix(node, node_name_to_scope, node_name_to_qconfig)
     activation_post_process = modules[node.target]
+    if hasattr(activation_post_process, "convert"):
+        activation_post_process.convert(model, node)
+        return
     # skip replacing observers to quant/dequant nodes if the qconfigs of all
     # consumers and producers of this observer are None
     skip_replacement = all(_has_none_qconfig(n, node_name_to_qconfig) for n in
@@ -131,8 +148,7 @@ def _replace_observer_with_quantize_dequantize_node_decomposed(
     if hasattr(activation_post_process, "is_dynamic"):
         is_dynamic = activation_post_process.is_dynamic  # type: ignore[assignment]
 
-    if dtype in [torch.quint8, torch.qint8, torch.qint32] and \
-            (not is_dynamic):
+    if dtype in SUPPORTED_QDTYPES and (not is_dynamic):
         # TODO: probably should cleanup this condition check, it's hard
         # to reason about this if and the following elif
 
@@ -204,6 +220,15 @@ def _replace_observer_with_quantize_dequantize_node_decomposed(
                 tuple(dq_inputs),
                 {}
             )
+
+            def remap_fn(x):
+                return dequantized_node if x is node else x
+
+            # remap numeric_debug_handle
+            for user_node in node.users:
+                if "numeric_debug_handle" in user_node.meta:
+                    numeric_debug_handle = user_node.meta["numeric_debug_handle"]
+                    user_node.meta["numeric_debug_handle"] = {remap_fn(k): v for k, v in numeric_debug_handle.items()}
             node.replace_all_uses_with(dequantized_node)
             graph.erase_node(node)
     elif is_dynamic:
@@ -303,6 +328,15 @@ def _replace_observer_with_quantize_dequantize_node_decomposed(
                 tuple(dq_inputs),
                 {}
             )
+
+            def remap_fn(x):
+                return dequantized_node if x is node else x
+
+            # remap numeric_debug_handle
+            for user_node in node.users:
+                if "numeric_debug_handle" in user_node.meta:
+                    numeric_debug_handle = user_node.meta["numeric_debug_handle"]
+                    user_node.meta["numeric_debug_handle"] = {remap_fn(k): v for k, v in numeric_debug_handle.items()}
             node.replace_all_uses_with(dequantized_node)
             graph.erase_node(node)
     elif dtype == torch.float16:
@@ -312,8 +346,7 @@ def _replace_observer_with_quantize_dequantize_node_decomposed(
     # activation_post_process is supported
 
 def _replace_observer_with_quantize_dequantize_node(
-        model: torch.nn.Module,
-        graph: Graph,
+        model: torch.fx.GraphModule,
         node: Node,
         modules: Dict[str, torch.nn.Module],
         node_name_to_scope: Dict[str, Tuple[str, type]],
@@ -328,6 +361,7 @@ def _replace_observer_with_quantize_dequantize_node(
     """
     assert modules is not None
     assert isinstance(node.target, str)
+    graph = model.graph
     module_path, prefix = _get_module_path_and_prefix(node, node_name_to_scope, node_name_to_qconfig)
     activation_post_process = modules[node.target]
     # skip replacing observers to quant/dequant nodes if the qconfigs of all
@@ -349,7 +383,7 @@ def _replace_observer_with_quantize_dequantize_node(
     if hasattr(activation_post_process, "is_dynamic"):
         is_dynamic = activation_post_process.is_dynamic  # type: ignore[attr-defined, assignment]
 
-    if dtype in [torch.quint8, torch.qint8, torch.qint32] and \
+    if dtype in [torch.quint8, torch.qint8, torch.qint32, torch.float8_e5m2, torch.float8_e4m3fn] and \
             (not is_dynamic):
         # TODO: probably should cleanup this condition check, it's hard
         # to reason about this if and the following elif
@@ -454,7 +488,7 @@ def _is_conversion_supported(activation_post_process: torch.nn.Module) -> bool:
         is_dynamic = activation_post_process.is_dynamic  # type: ignore[attr-defined, assignment]
 
     return (
-        (dtype in [torch.quint8, torch.qint8, torch.qint32] and (not is_dynamic)) or  # type: ignore[return-value]
+        (dtype in SUPPORTED_QDTYPES and (not is_dynamic)) or  # type: ignore[return-value]
         is_dynamic or
         dtype == torch.float16
     )
@@ -915,24 +949,33 @@ def convert(
     if convert_custom_config is None:
         convert_custom_config = ConvertCustomConfig()
 
-    if isinstance(convert_custom_config, Dict):
+    if isinstance(convert_custom_config, dict):
         warnings.warn(
             "Passing a convert_custom_config_dict to convert is deprecated and will not be supported "
-            "in a future version. Please pass in a ConvertCustomConfig instead.")
+            "in a future version. Please pass in a ConvertCustomConfig instead.",
+            FutureWarning,
+            stacklevel=2,
+        )
         convert_custom_config = ConvertCustomConfig.from_dict(convert_custom_config)
 
-    if isinstance(qconfig_mapping, Dict):
+    if isinstance(qconfig_mapping, dict):
         warnings.warn(
             "Passing a QConfig dictionary to convert is deprecated and will not be supported "
-            "in a future version. Please pass in a QConfigMapping instead.")
+            "in a future version. Please pass in a QConfigMapping instead.",
+            FutureWarning,
+            stacklevel=2,
+        )
         qconfig_mapping = QConfigMapping.from_dict(qconfig_mapping) if qconfig_mapping else None
     qconfig_mapping = copy.deepcopy(qconfig_mapping)
-    assert(qconfig_mapping is None or isinstance(qconfig_mapping, QConfigMapping))
+    assert qconfig_mapping is None or isinstance(qconfig_mapping, QConfigMapping)
 
-    if isinstance(backend_config, Dict):
+    if isinstance(backend_config, dict):
         warnings.warn(
             "Passing a backend_config_dict to prepare is deprecated and will not be supported "
-            "in a future version. Please pass in a BackendConfig instead.")
+            "in a future version. Please pass in a BackendConfig instead.",
+            FutureWarning,
+            stacklevel=2,
+        )
         backend_config = BackendConfig.from_dict(backend_config)
 
     if backend_config is None:
@@ -1054,11 +1097,11 @@ def convert(
                 else:
                     if is_decomposed:
                         _replace_observer_with_quantize_dequantize_node_decomposed(
-                            model, model.graph, node, modules, node_name_to_scope,
+                            model, node, modules, node_name_to_scope,
                             node_name_to_qconfig)
                     else:
                         _replace_observer_with_quantize_dequantize_node(
-                            model, model.graph, node, modules, node_name_to_scope,
+                            model, node, modules, node_name_to_scope,
                             node_name_to_qconfig)
             elif isinstance(mod, DeQuantStub):
                 _replace_observer_or_dequant_stub_with_dequantize_node(node, model.graph)
