@@ -304,7 +304,7 @@ void SourceImporterImpl::importNamedType(
   }
 }
 
-c10::optional<Assign> SourceImporterImpl::
+std::optional<Assign> SourceImporterImpl::
     attributeAssignmentSpecialHandlingHack(
         const QualifiedName& qualified_classname,
         const Assign& assign) {
@@ -439,6 +439,12 @@ void SourceImporterImpl::importClass(
     switch (statement.kind()) {
       case TK_ASSIGN: {
         const auto assign = Assign(statement);
+        auto check_assign_values = [&assign](const std::string& name) {
+          TORCH_CHECK(
+              assign.rhs().present(),
+              "Malformed assignment statement: missing values to assign in ",
+              name);
+        };
         switch (assign.lhs().kind()) {
           case TK_VAR: {
             const auto name = Var(assign.lhs()).name().name();
@@ -451,6 +457,7 @@ void SourceImporterImpl::importClass(
                   is_module,
                   "Assignments in class body only "
                   "supported on modules right now");
+              check_assign_values(name);
               const auto param_list = ListLiteral(assign.rhs().get()).inputs();
               for (const auto& param : param_list) {
                 parameter_names.insert(StringLiteral(param).text());
@@ -461,6 +468,7 @@ void SourceImporterImpl::importClass(
             } else if (name == "__buffers__") {
               TORCH_INTERNAL_ASSERT(
                   is_module, "Buffers only exist on modules at the moment");
+              check_assign_values(name);
               const auto buffer_list = ListLiteral(assign.rhs().get()).inputs();
               for (const auto& buffer : buffer_list) {
                 buffer_names.insert(StringLiteral(buffer).text());
@@ -469,6 +477,7 @@ void SourceImporterImpl::importClass(
               TORCH_INTERNAL_ASSERT(
                   is_module,
                   "Forward pre hooks only exist on modules at the moment");
+              check_assign_values(name);
               const auto pre_hook_list =
                   ListLiteral(assign.rhs().get()).inputs();
               for (const auto& pre_hook : pre_hook_list) {
@@ -480,6 +489,7 @@ void SourceImporterImpl::importClass(
               TORCH_INTERNAL_ASSERT(
                   is_module,
                   "Forward hooks only exist on modules at the moment");
+              check_assign_values(name);
               const auto hook_list = ListLiteral(assign.rhs().get()).inputs();
               for (const auto& hook : hook_list) {
                 std::string hook_name = StringLiteral(hook).text();
@@ -693,7 +703,7 @@ void SourceImporterImpl::importNamedTuple(
     const auto assign = Assign(statement);
 
     auto name = Var(Assign(statement).lhs()).name().name();
-    c10::optional<IValue> default_val;
+    std::optional<IValue> default_val;
     if (assign.rhs().present()) {
       std::vector<IValue> parsed = type_parser.evaluateDefaults(
           assign.rhs().range(), {assign.rhs().get()}, {assign.type().get()});
